/**
 * Copyright (C) 2001 Billy Biggs <vektor@dumbterm.net>.
 *
 * Mixer routines stolen from mplayer, http://mplayer.sourceforge.net.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include "ossinput.h"

#define OUTPUT_BUF_SIZE  65536

static int audio_fd = -1;
static int outputsize = OUTPUT_BUF_SIZE;

int oss_read_open( int bits, int rate, int channels )
{
    int format = AFMT_S16_LE;
    int chanreturn = channels - 1;
    int numfrags, fragsize, frag;
    int trigger;
    audio_buf_info info;

    fprintf( stderr, "ossinput: Opening %d bit audio, %dkhz, %d channels.\n",
             bits, rate, channels );
    audio_fd = open( "/dev/dsp", O_RDONLY );
    if( audio_fd < 0 ) {
        perror( "soundcard open" );
    }

    fprintf( stderr, "ossinput: Soundcard successfully opened.\n" );

    numfrags = 4;
    fragsize = 8192;

    frag = 0;
    while( fragsize > 0 ) {
        fragsize /= 2;
        frag++;
    }
    frag--;

    frag = ( ( ( numfrags - 1 ) << 16 ) | frag );
    if( ioctl( audio_fd, SNDCTL_DSP_SETFRAGMENT, &frag ) ) {
        perror( "set fragments" );
    }

    if( ioctl( audio_fd, SNDCTL_DSP_SETFMT, &format ) == -1 ) {
        perror( "set format" );
    }

    if( ioctl( audio_fd, SNDCTL_DSP_SPEED, &rate ) == -1 ) {
        perror( "set rate" );
    }

    if( ioctl( audio_fd, SNDCTL_DSP_STEREO, &chanreturn ) == -1 ) {
        perror( "set channels" );
    }

    if( ioctl( audio_fd, SNDCTL_DSP_GETISPACE, &info ) == -1 ) {
        perror( "get info" );
    }

    if( ioctl( audio_fd, SNDCTL_DSP_GETBLKSIZE, &fragsize ) == -1 ) {
        perror( "get info" );
    }

    if( info.fragsize < OUTPUT_BUF_SIZE ) {
        outputsize = info.fragsize;
    }

    fprintf( stderr, "ossinput: dsp opened with %d fragments of size %d (%d).\n",
             info.fragments, info.fragsize, fragsize );

    /* Trigger record. */
    trigger = ~PCM_ENABLE_INPUT;
    ioctl(audio_fd,SNDCTL_DSP_SETTRIGGER,&trigger);

    trigger = PCM_ENABLE_INPUT;
    ioctl(audio_fd,SNDCTL_DSP_SETTRIGGER,&trigger);

    return info.fragsize;
}

int oss_read_ready( void )
{
    struct timeval tv;
    fd_set readfds;

    FD_ZERO( &readfds );
    FD_SET( audio_fd, &readfds );

    tv.tv_sec = 0;
    tv.tv_usec = 0;

    return ( select( audio_fd + 1, &readfds, 0, 0, &tv ) > 0 );
}

size_t oss_read_fragment( unsigned char *samples, int size, int *fragnum )
{
    count_info info;
    size_t ret;

    ret = read( audio_fd, samples, size );
    ioctl( audio_fd, SNDCTL_DSP_GETIPTR, &info );
    *fragnum = ( info.bytes - (info.ptr & 8191 ) ) >> 13;

    /*
    fprintf( stderr, "bytes: %d:%d:%d, fragments: %d\n",
               info.blocks, info.bytes, info.ptr, (info.bytes - (info.ptr%8192)) / 8192 );
    */

    return ret;
}

/*
int oss_get_bytes( void )
{
    count_info info;
    ioctl( audio_fd, SNDCTL_DSP_GETOPTR, &info );
    // return ( num_ive_written - info.bytes );
    return num_ive_written;
}
*/

void oss_read_close( void )
{
    if( audio_fd >= 0 ) {
        close( audio_fd );
    }
}

